@@ -19,15 +19,19 @@
 */
 
 #include "karchivetest.h"
+#include <kar.h>
 #include <ktar.h>
 #include <kzip.h>
 #include <k7zip.h>
 #include <krcc.h>
 
-#include <QtTest>
+#include <QTest>
+#include <QTextCodec>
+#include <QSaveFile>
 #include <QFileInfo>
 #include <kfilterdev.h>
 #include <qtemporarydir.h>
+#include <QRegularExpression>
 
 #ifndef Q_OS_WIN
 #include <unistd.h> // symlink
@@ -44,6 +48,13 @@
 
 QTEST_MAIN(KArchiveTest)
 
+void initLocale()
+{
+    // KArchive uses QFile::decodeName, and our tests use utf8 encoding for filenames
+    QTextCodec::setCodecForLocale(QTextCodec::codecForName("utf-8"));
+}
+Q_CONSTRUCTOR_FUNCTION(initLocale)
+
 static const int SIZE1 = 100;
 
 /**
@@ -136,7 +147,7 @@ static QStringList recursiveListEntries(const KArchiveDirectory *dir, const QStr
     QStringList ret;
     QStringList l = dir->entries();
     l.sort();
-    Q_FOREACH (const QString &it, l) {
+    for (const QString &it : qAsConst(l)) {
         const KArchiveEntry *entry = dir->entry(it);
 
         QString descr;
@@ -216,6 +227,7 @@ static void testFileData(KArchive *archive)
     dev->seek(4);
     contents = dev->read(1);
     QCOMPARE(contents, QByteArray("o"));
+    delete dev;
 
     const KArchiveEntry *e = dir->entry(QStringLiteral("mediumfile"));
     QVERIFY(e && e->isFile());
@@ -305,7 +317,7 @@ static void testCopyTo(KArchive *archive)
     const QFileInfo fileInfo5(fileName);
     QVERIFY(fileInfo5.exists());
     QVERIFY(fileInfo5.isFile());
-    // Do not use fileInfo.readLink() for unix symlinks
+    // Do not use fileInfo.symLinkTarget() for unix symlinks
     // It returns the -full- path to the target, while we want the target string "as is".
     QString symLinkTarget;
     const QByteArray encodedFileName = QFile::encodeName(fileName);
@@ -484,7 +496,7 @@ void KArchiveTest::testReadTar() // testCreateTarGz must have been run first.
     const QString owner = localFileData.owner();
     const QString group = localFileData.group();
     const QString emptyTime = QDateTime().toString(QStringLiteral("dd.MM.yyyy hh:mm:ss"));
-    const QDateTime creationTime = QFileInfo(fileName).created();
+    const QDateTime creationTime = QFileInfo(fileName).birthTime();
 
     // 1.6-1.7 ms per interaction, 2908428 instruction loads
     // After the "no tempfile when writing fix" this went down
@@ -529,13 +541,13 @@ void KArchiveTest::testReadTar() // testCreateTarGz must have been run first.
 
         // This one was added with addLocalFile, so ignore mode.
         QString str = listing[14];
-        str.replace(QRegExp(QStringLiteral("mode.*user=")), QStringLiteral("user="));
+        str.replace(QRegularExpression(QStringLiteral("mode.*user=")), QStringLiteral("user="));
 
         compareEntryWithTimestamp(str, QString("user=%1 group=%2 path=z/test3 type=file size=13").arg(owner).arg(group), creationTime);
 
 #ifndef Q_OS_WIN
         str = listing[15];
-        str.replace(QRegExp(QStringLiteral("mode.*path=")), QStringLiteral("path="));
+        str.replace(QRegularExpression(QStringLiteral("mode.*path=")), QStringLiteral("path="));
 
         compareEntryWithTimestamp(str, QString("path=z/test3_symlink type=file size=0 symlink=test3"), creationTime);
 #endif
@@ -751,6 +763,24 @@ void KArchiveTest::testTarDirectoryForgotten()
     QVERIFY(tar.close());
 }
 
+void KArchiveTest::testTarEmptyFileMissingDir()
+{
+    KTar tar(QFINDTESTDATA(QLatin1String("tar_emptyfile_missingdir.tar.gz")));
+    QVERIFY(tar.open(QIODevice::ReadOnly));
+
+    const KArchiveDirectory *dir = tar.directory();
+    QVERIFY(dir != nullptr);
+
+    const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
+
+    QCOMPARE(listing[0], QString("mode=40777 path=dir type=dir"));
+    QCOMPARE(listing[1], QString("mode=40777 path=dir/foo type=dir"));
+    QCOMPARE(listing[2], QString("mode=644 path=dir/foo/file type=file size=0"));
+    QCOMPARE(listing.count(), 3);
+
+    QVERIFY(tar.close());
+}
+
 void KArchiveTest::testTarRootDir() // bug 309463
 {
     KTar tar(QFINDTESTDATA(QLatin1String("tar_rootdir.tar.gz")));
@@ -773,8 +803,7 @@ void KArchiveTest::testTarLongNonASCIINames() // bug 266141
 {
     const QString tarName = QString("karchive-long-non-ascii-names.tar");
     const QString longName =
-        QString("раз-два-три-четыре-пять-вышел-зайчик-погулять-вдруг-охотник-"
-                "выбегает-прямо-в-зайчика.txt");
+        QString::fromUtf8("раз-два-три-четыре-пять-вышел-зайчик-погулять-вдруг-охотник-выбегает-прямо-в-зайчика.txt");
 
     {
         KTar tar(tarName);
@@ -980,11 +1009,11 @@ void KArchiveTest::testReadZip()
     QCOMPARE(listing[14], QString("mode=40777 path=z type=dir"));
     // This one was added with addLocalFile, so ignore mode
     QString str = listing[15];
-    str.replace(QRegExp(QStringLiteral("mode.*path=")), QStringLiteral("path="));
+    str.replace(QRegularExpression(QStringLiteral("mode.*path=")), QStringLiteral("path="));
     QCOMPARE(str, QString("path=z/test3 type=file size=13"));
 #ifndef Q_OS_WIN
     str = listing[16];
-    str.replace(QRegExp(QStringLiteral("mode.*path=")), QStringLiteral("path="));
+    str.replace(QRegularExpression(QStringLiteral("mode.*path=")), QStringLiteral("path="));
     QCOMPARE(str, QString("path=z/test3_symlink type=file size=5 symlink=test3"));
 #endif
 
@@ -1052,7 +1081,7 @@ void KArchiveTest::testZipWithNonLatinFileNames()
     QVERIFY(zip.open(QIODevice::WriteOnly));
 
     const QByteArray fileData("Test of data with a russian file name");
-    const QString fileName = QStringLiteral("Архитектура.okular");
+    const QString fileName = QString::fromUtf8("Архитектура.okular");
     const QString recodedFileName = QFile::decodeName(QFile::encodeName(fileName));
     QVERIFY(zip.writeFile(fileName, fileData));
 
@@ -1246,6 +1275,42 @@ void KArchiveTest::testRcc()
     QCOMPARE(QString::fromLatin1(fileData), QString::fromLatin1("root\n"));
 }
 
+void KArchiveTest::testAr()
+{
+    const QString artestFile = QFINDTESTDATA("data/artest.a");
+    KAr ar(artestFile);
+
+    QVERIFY(ar.open(QIODevice::ReadOnly));
+
+    const KArchiveDirectory *dir = ar.directory();
+    QVERIFY(dir != nullptr);
+
+    const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
+
+    const QStringList expected = {
+            "mode=100644 path=at_quick_exit.oS type=file size=3456",
+            "mode=100644 path=atexit.oS type=file size=3436",
+            "mode=100644 path=elf-init.oS type=file size=3288",
+            "mode=100644 path=fstat.oS type=file size=3296",
+            "mode=100644 path=fstat64.oS type=file size=3304",
+            "mode=100644 path=fstatat.oS type=file size=3352",
+            "mode=100644 path=fstatat64.oS type=file size=3380",
+            "mode=100644 path=lstat.oS type=file size=3320",
+            "mode=100644 path=lstat64.oS type=file size=3332",
+            "mode=100644 path=mknod.oS type=file size=2840",
+            "mode=100644 path=mknodat.oS type=file size=2848",
+            "mode=100644 path=stack_chk_fail_local.oS type=file size=2288",
+            "mode=100644 path=stat.oS type=file size=3316",
+            "mode=100644 path=stat64.oS type=file size=3312",
+            "mode=100644 path=warning-nop.oS type=file size=1976"
+    };
+
+    QCOMPARE(listing.count(), expected.count());
+    QCOMPARE(listing, expected);
+
+    QVERIFY(ar.close());
+}
+
 /**
  * @see QTest::cleanupTestCase()
  */
@@ -1335,11 +1400,11 @@ void KArchiveTest::testRead7Zip() // testCreate7Zip must have been run first.
         QCOMPARE(listing[13], QString("mode=40777 path=z type=dir"));
         // This one was added with addLocalFile, so ignore mode/user/group.
         QString str = listing[14];
-        str.replace(QRegExp(QStringLiteral("mode.*path=")), QStringLiteral("path="));
+        str.replace(QRegularExpression(QStringLiteral("mode.*path=")), QStringLiteral("path="));
         QCOMPARE(str, QString("path=z/test3 type=file size=13"));
 #ifndef Q_OS_WIN
         str = listing[15];
-        str.replace(QRegExp(QStringLiteral("mode.*path=")), QStringLiteral("path="));
+        str.replace(QRegularExpression(QStringLiteral("mode.*path=")), QStringLiteral("path="));
         QCOMPARE(str, QString("path=z/test3_symlink type=file size=0 symlink=test3"));
 #endif
 
